/******************************************************************************
 *
 * Program: CPU Load NT plugin for Nagios (nrpe_nt).
 * License: GPL
 *
 * Designed as a Nagios Plugin (based on plugins by Ethan Galstad).
 * Copyright (C) 2004 Stephen Strudwick (Pipex Communications, sas@pipex.net)
 *
 * License Information:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *****************************************************************************/

/*
 * Some lines of code are based or taken from the NetSaintService for NT
 * which is Copyright (c) 2000 Jan Christian Kaldestad which as far as I can tell
 * is freeware.
 */

#include "stdafx.h"
#include <windows.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <io.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "cpuusage.h"

const int POLLS = 20;
const int OK = 0;
const int Warning = 1;
const int Critical = 2;
const int Unknown = 3;

int main(int argc, char* argv[])
{
	int warning_threshold;
	int critical_threshold;	
    cpuusage	CPU;

	/* Get Values from command line */
	if (argc != 3) {
      printf ("NRPE_NT Plugin, CPU Load\n");
      printf ("usage: cpuload warning threshold(%%) critical threshold(%%)\n");
      printf ("example: 70 90\n");
      return Unknown;
	}

    warning_threshold = atoi(argv[1]);
	critical_threshold = atoi(argv[2]);

	/* Check cpu */
	// Poll
	int n;
    float mean, variance;
    float cmean, cvariance;

	for(n=0;n<POLLS;n++) {
	  CPU.poll();
	  Sleep(100);
	}

	/* Print results */
	mean = CPU.mean(POLLS);

	variance = CPU.variance(POLLS, mean);
	cmean = CPU.mean(MAX_QUEUE_HISTORY);
	cvariance = CPU.variance(MAX_QUEUE_HISTORY, cmean);

	printf("NOW: Mean:%f%s Variance: %f%s ", mean, "%", (variance / 2500) * 100, "%");
    printf("CUMULATIVE: Mean:%f%s Variance: %f%s\n", cmean, "%", (cvariance / 2500) * 100, "%");

	/* return on correct value depending on result and warning/critical params */
	if (mean >= critical_threshold)
		return Critical;

	if (mean >= warning_threshold)
		return Warning;

	return OK;
}