/******************************************************************************
 *
 * Program: CPU Load NT plugin for Nagios (nrpe_nt).
 * License: GPL
 *
 * Designed as a Nagios Plugin (based on plugins by Ethan Galstad).
 * Copyright (C) 2004 Stephen Strudwick (Pipex Communications, sas@pipex.net)
 *
 * License Information:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *****************************************************************************/

/* A FILO queue implementation */

#include "stdafx.h"
#include "queue.h"
#include <stdlib.h>
#include "common.h"

#define MinQueueSize ( 5 )

struct QueueRecord {
  int Capacity;
  int Front;
  int Rear;
  int Size;
  ElementType *Array;
};

int IsEmpty( Queue Q ) {
  return Q->Size == 0;
}

int IsFull( Queue Q ) {
  return Q->Size == Q->Capacity;
}

Queue CreateQueue( int MaxElements ) {
  Queue Q;

  if( MaxElements < MinQueueSize ) {
    debug("Queue size is too small");
  	return 0;
  }

  Q = (QueueRecord *)malloc( sizeof( struct QueueRecord ) );
  if( Q == NULL ) {
	 debug("Out of space for Queue!!!");
	 return 0;
  }

  Q->Array = (int *)malloc( sizeof( ElementType ) * MaxElements );
  if( Q->Array == NULL ) {
	debug("Out of space for Queue!!!");
	return 0;
  }

  Q->Capacity = MaxElements;

  MakeEmpty(Q);
  LoadQueue(Q);

  return Q;
}

void MakeEmpty( Queue Q ) {
  Q->Size = 0;
  Q->Front = 1;
  Q->Rear = 0;
}


int LoadQueue(Queue Q) {
	FILE *file;
	char *pbuf;
	char buf[257];

	debug("loading");
	if (!(file = fopen ("cpuload.tmp", "r"))) {
      return -1;
    }
    else {

	  while(pbuf = fgets(buf, 256, file)) {
		  Enqueue(atoi(buf),Q);
	  }

      fclose (file);
    }

	return 0;
}

int SaveQueue(Queue Q) {
	FILE *file;
	int n;

	debug("saving");
	if (!(file = fopen ("cpuload.tmp", "w"))) {
      return -1;
    }
    else {

	  for(n=Q->Front;n<=Q->Size;n++) {
		  fprintf (file, "%d\n", Q->Array[n]);
	  }

      fclose (file);
    }

	return 0;
}

void DisposeQueue( Queue Q ) {
  if( Q != NULL ) {
	/* first save data to disk */
    SaveQueue(Q);
    free( Q->Array );
    free( Q );
  }
}

static int Succ( int Value, Queue Q ) {
  if( ++Value == Q->Capacity )
    Value = 0;
    return Value;
}

void Enqueue( ElementType X, Queue Q ) {
  if (X < 0 || X > 100) {
	debug("Value out of range");
	return;
  }

  if( IsFull( Q ) ) {
    debug("Full queue");
	return;
  } else {
    Q->Size++;
	Q->Rear = Succ( Q->Rear, Q );
	Q->Array[ Q->Rear ] = X;
  }
}

ElementType Front( Queue Q ) {
  if( !IsEmpty( Q ) ) {
	debug("Empty queue");
    return Q->Array[ Q->Front ];
  }
  return 0;
}

void Dequeue( Queue Q ) {
  if( IsEmpty( Q ) ) {
 	debug("Empty queue");
	return;
  }

  else {
    Q->Size--;
    Q->Front = Succ( Q->Front, Q );
  }
}

int Size(Queue Q) {
  return Q->Size;
}

int Rear(Queue Q) {
  return Q->Rear;
}

int Element(int position, Queue Q) {
  return Q->Array[position];
}

ElementType FrontAndDequeue( Queue Q ) {
  ElementType X = 0;

  if( IsEmpty( Q ) ) {
	debug("Empty queue");
	return 0;
  } else {
    Q->Size--;
    X = Q->Array[ Q->Front ];
    Q->Front = Succ( Q->Front, Q );
  }
  return X;
}