/******************************************************************************
 *
 * Program: Disk Space NT plugin for Nagios (nrpe_nt).
 * License: GPL
 *
 * Designed as a Nagios Plugin (based on plugins by Ethan Galstad).
 * Copyright (C) 2004 Stephen Strudwick (Pipex Communications, sas@pipex.net)
 *
 * License Information:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *****************************************************************************/

#include "stdafx.h"
#include "disks.h"

bool getDiskSpace(const char* disk_name, ULARGE_INTEGER& all_disk, ULARGE_INTEGER& avail_disk)
{
	int err;
	ULARGE_INTEGER temp_disk_caller;
	ULARGE_INTEGER temp_disk_space;
	ULARGE_INTEGER temp_disk_avail;

	debug("Finding available disk space on: '%s'", disk_name);

	err = GetDiskFreeSpaceEx(disk_name, &temp_disk_caller, &temp_disk_space, &temp_disk_avail);
	if(!err)
	{
		printToConsole("GetDiskFreeSpaceEx() returned an error" );
		return false;
	}
	
	// convert to KB and copy out
    all_disk.QuadPart = temp_disk_space.QuadPart;
	avail_disk.QuadPart = temp_disk_avail.QuadPart;
	return true;
}

disk_result CheckDiskUsage(char *path)
{
  disk_result disk;
  disk.free_mb = 0;
  disk.total_mb = 0;
  disk.used_mb = 0;

  // Real memory
  ULARGE_INTEGER all_disk;
  ULARGE_INTEGER avail_disk;
  if (!getDiskSpace(path, all_disk, avail_disk))
  {
    return disk;
  }
  
  int free_mb = (int)(avail_disk.QuadPart >> 20);
  int total_mb = (int)(all_disk.QuadPart >> 20);
  int used_mb = total_mb - free_mb;

  disk.free_mb = free_mb;
  disk.used_mb = used_mb;
  disk.total_mb = total_mb;

  return disk;
}