/******************************************************************************
 *
 * Program: Disk Space NT plugin for Nagios (nrpe_nt).
 * License: GPL
 *
 * Designed as a Nagios Plugin (based on plugins by Ethan Galstad).
 * Copyright (C) 2004 Stephen Strudwick (Pipex Communications, sas@pipex.net)
 *
 * License Information:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *****************************************************************************/

/*
 * Some lines of code are based or taken from the NetSaintService for NT
 * which is Copyright (c) 2000 Jan Christian Kaldestad which as far as I can tell
 * is freeware.
 */

#include "stdafx.h"
#include "disks.h"
#include <windows.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <io.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>

const int OK = 0;
const int Warning = 1;
const int Critical = 2;
const int Unknown = 3;

int main(int argc, char* argv[])
{
	int warning_threshold;
	int critical_threshold;
    char drive[257];
	disk_result disk;

	/* Get Values from command line */
	if (argc != 4) {
      printf ("NRPE_NT Plugin, Disk Load\n");
      printf ("usage: diskload drive warning threshold(%%) critical threshold(%%)\n");
      printf ("example: D: 77 90\n");
      return Unknown;
	}

	strncpy(drive,argv[1],256);  /* Get drive */
    warning_threshold = atoi(argv[2]);
	critical_threshold = atoi(argv[3]);

    if (!drive) {
	  printf ("drive invalid\n");
	  return Unknown;
	} 

	/* Check disk space */
	disk = CheckDiskUsage(drive);

	/* Print results */
	/* Work out pecentage used/free */
    int percent_used = (disk.used_mb / (double)disk.total_mb) * 100;
	int percent_free = (disk.free_mb / (double)disk.total_mb) * 100;
  
	printf("Used: %d MB (%d%%) Free: %d MB (%d%%)\n", disk.used_mb, percent_used, disk.free_mb, percent_free);

	/* return on correct value depending on result and warning/critical params */
	if (percent_used >= critical_threshold)
		return Critical;

	if (percent_used >= warning_threshold)
		return Warning;

	return OK;
}

