/******************************************************************************
 *
 * Program: Eventlog plugin for Nagios (nrpe_nt).
 * License: GPL
 *
 * Designed as a Nagios Plugin (based on plugins by Ethan Galstad).
 * Copyright (C) 2004 Stephen Strudwick (Pipex Communications, sas@pipex.net)
 *
 * License Information:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *****************************************************************************/

#include "stdafx.h"
#include "eventLog.h"
#include "common.h"

const int MAXSIZE = 1000;

using namespace std;
typedef list<SourceData> SourceDataList;


int GetNewRecords( char*			logName, 
                  SourceDataList	&srcList,				   
                  int		nSec );


void AddEventSource(char *installDir)
{
  HKEY hk; 
  DWORD dwData; 
  char*	szBuf;
  int		bufSize;	
  char*	regKey = "SYSTEM\\CurrentControlSet\\Services\\EventLog\\Application\\";
  
  if(strlen(regKey) > strlen(installDir))
  {
    bufSize =   strlen(regKey) + strlen(SZAPPNAME);	
  }
  else
  {
    bufSize =   strlen(installDir+1) + strlen(SZAPPNAME);	
  }
  
  szBuf = new char[bufSize+128];
  
  strcpy(szBuf, regKey);	
  strcat(szBuf, SZAPPNAME);
  
  // Add your source name as a subkey under the Application 
  // key in the EventLog registry key. 
  
  if (RegCreateKey(HKEY_LOCAL_MACHINE, szBuf, &hk))
  {
    debug("Could not create the registry key");
    delete [] szBuf;
    szBuf = 0;
    return;
  }     
  
  // Set the name of the message file
  
  strcpy(szBuf, installDir);
  strcat(szBuf, "\\");
  strcat(szBuf, SZAPPNAME);
  strcat(szBuf, ".exe");
  
  // Add the name to the EventMessageFile subkey. 
  
  if (RegSetValueEx(hk,             // subkey handle 
    "EventMessageFile",       // value name 
    0,                        // must be zero 
    REG_EXPAND_SZ,            // value type 
    (LPBYTE) szBuf,      // pointer to value data 
    strlen(szBuf) + 1))  // length of value data 
  {
    debug("Could not set the event message file");
    RegCloseKey(hk);
    delete [] szBuf;
    szBuf = 0;
    return;	
  }       
  
  // Set the supported event types in the TypesSupported subkey. 
  
  dwData = EVENTLOG_ERROR_TYPE | EVENTLOG_WARNING_TYPE | 
    EVENTLOG_INFORMATION_TYPE; 
  
  if (RegSetValueEx(hk,      // subkey handle 
    "TypesSupported",  // value name 
    0,                 // must be zero 
    REG_DWORD,         // value type 
    (LPBYTE) &dwData,  // pointer to value data 
    sizeof(DWORD)))    // length of value data 
  {
    debug("ERR: Could not set the supported types");
  }	
  
  delete [] szBuf;
  szBuf = 0;
  
  RegCloseKey(hk);
} 

int GetNewRecords( char*			logName, 
                  SourceDataList	&srcList,				   
                  int			nSec )
                  
{	
  SourceDataList::iterator it;
  
  HANDLE			h;
  EVENTLOGRECORD	*ptr; 
  BYTE				bBuffer[1000]; 
  DWORD				dwRead, dwNeeded, dwThisRecord = 0; 
  bool				stop=false;
  bool				firstTime = true;
  bool				listUpdated=false;	
  int				nNewRecords = 0;  
  int				bytesRequired=0;
  SourceData		srcData;    
  srcData.message = NULL;	
  int				nListElements=0;
  time_t			dwTime;
  
  h = OpenEventLog( NULL, logName);   // source name 
  if (h == NULL) {
    return -1;
  }
  
  ptr = (EVENTLOGRECORD *) &bBuffer;

  time( &dwTime );
  dwTime -= nSec;

  while (ReadEventLog( h,							// event log handle 
	EVENTLOG_BACKWARDS_READ |
    EVENTLOG_SEQUENTIAL_READ,	// sequential read 
    0,							// ignored for sequential reads 
    ptr,						// pointer to buffer 
    1000,						// size of buffer 
    &dwRead,					// number of bytes read 
    &dwNeeded) && !stop)					// bytes in next record
  {
    while (dwRead > 0)
    {
      // Only items that are new enough
      if( dwTime <= ptr->TimeGenerated) {				
        strncpy(srcData.sourcName, (LPSTR) ((LPBYTE) ptr + sizeof(EVENTLOGRECORD)), 255);
        srcData.EventType = ptr->EventType;
        srcData.eventID = ptr->EventID;	
        srcData.genTime = ptr->TimeGenerated;
        
        // run through the list and check if the combination
        // sourceName - eventId already exist
        for (it = srcList.begin(); it != srcList.end(); ++it)
        {
          if( strcmp(it->sourcName, srcData.sourcName) == 0
            && it->eventID == srcData.eventID )
          {					
            switch(srcData.EventType)
            {
            case EVENTLOG_ERROR_TYPE:							
              it->nErr++;							
              break;
            case EVENTLOG_WARNING_TYPE:			
              it->nWarn++;
              break;
            case EVENTLOG_INFORMATION_TYPE:				
              it->nInf++;					
              break;
            case EVENTLOG_AUDIT_SUCCESS:			
              it->nAudS++;
              break;
            case EVENTLOG_AUDIT_FAILURE:			
              it->nAudF++;
              break;						
            }
            
            it->nMessages++;							
            listUpdated = true;											
          }
        }				
        
        // A new message with a new id has been found
        if(!listUpdated) 
        {
          
          srcData.nErr = srcData.nWarn = srcData.nInf = 0;
          srcData.nAudF = srcData.nAudS = 0;								
          
          switch(srcData.EventType)
          {
          case EVENTLOG_ERROR_TYPE:
            srcData.nErr++;
            break;
          case EVENTLOG_WARNING_TYPE:			
            srcData.nWarn++;
            break;
          case EVENTLOG_INFORMATION_TYPE:				
            srcData.nInf++;
            break;
          case EVENTLOG_AUDIT_SUCCESS:			
            srcData.nAudS++;
            break;
          case EVENTLOG_AUDIT_FAILURE:			
            srcData.nAudF++;
            break;			
          }
          
          srcData.nMessages = 1;
          srcList.insert (srcList.begin(), srcData);
          it = srcList.begin();
          
          nListElements++;
          
        }
        
        listUpdated = false;				
        nNewRecords++;
	  } else {
		stop=true;
        break;
	  }			
      dwRead -= ptr->Length;
      ptr = (EVENTLOGRECORD *) ((LPBYTE) ptr + ptr->Length);
	} 
    
    ptr = (EVENTLOGRECORD *) &bBuffer;
  }	
  
  CloseEventLog(h);
  return nNewRecords;
}

int not_exluded(char * string, Record R) {
  int n = 0;
  for(n=0;n<R->Size;n++) {
	  if(strcmp(string,R->Array[n])==0){
		  /* excluded */
		  return 0;
	  }
  }

  /* not */
  return 1;
}

void build_record(Record R, char *exclude) {
  // put exlude list into an array of char *
  // add function that checks if string in array
  /* split and loop over string */
  int n = 0;
  int ptr = 0;
  int len = strlen(exclude);
  char buf[257];


  /* Loop until NULL */
  for(n=0;n<len+1;n++) {
  /* Go till we hit a comma */
    if (exclude[n] != ',' && exclude[n] != '\0') {
		buf[ptr] = exclude[n];
		ptr++;
	} else if (ptr) {
		/* Got null terminated string */
		buf[ptr] = '\0';
		/* Reset pointer */
		ptr = 0;
		if (R->Size < R->Capacity) {
		  R->Array[R->Size] = strdup(buf);
		  R->Size++;
		}
	}
  }
}

int CheckEventLog(int period, char *exclude, char *exclude_id)
{
  int				nNewRecords=0;
  int				nBytesNeeded=0;	
  int				index = 0;
  int				ret=OK;
  int				n = 0;
  char				eventid[100];
  SourceDataList srcDataList;
  SourceDataList::iterator it;

  GetNewRecords( "Application", srcDataList, period*60 );
  GetNewRecords( "Security", srcDataList, period*60 );		
  GetNewRecords( "System", srcDataList, period*60 );
  
  Record			R;
  Record			R_id;

  R = (QueueRecord *)malloc( sizeof( struct QueueRecord ) );
  R->Array = (char **)malloc( sizeof( char ** ) * MAXSIZE );

  R_id = (QueueRecord *)malloc( sizeof( struct QueueRecord ) );
  R_id->Array = (char **)malloc( sizeof( char ** ) * MAXSIZE );


  R->Capacity = MAXSIZE;
  R->Size = 0;

  R_id->Capacity = MAXSIZE;
  R_id->Size = 0;

  build_record(R,exclude);
  build_record(R_id,exclude_id);

  /* print out every warning or error */
  /* set return value */
  /* ignore if it->sourcName matches item in exlude list */;

  for (it = srcDataList.begin(); it != srcDataList.end(); ++it)
  {

	  _snprintf(eventid, 99, "%lu", it->eventID& 0x0000FFFF);

	  if (not_exluded(it->sourcName, R) && not_exluded(eventid, R_id)) {
		if (it->nErr || it->nWarn) {
			printf("%s(%s): Err=%d Warn=%d Instances=%d ", it->sourcName, eventid, it->nErr, it->nWarn, it->nMessages);
		}
		if (it->nWarn) {
			ret=Warning;
		}
		if (it->nErr) {
			ret=Critical;
		}
	  }
  }
  
  srcDataList.clear();

  for(n=0;n<R->Size;n++) {
	  free(R->Array[n]);
  }
  free(R->Array);
  free(R);

  for(n=0;n<R_id->Size;n++) {
	  free(R_id->Array[n]);
  }
  free(R_id->Array);
  free(R_id);


  return ret;
}