/******************************************************************************
 *
 * Program: Memory Load plugin for Nagios (nrpe_nt).
 * License: GPL
 *
 * Designed as a Nagios Plugin (based on plugins by Ethan Galstad).
 * Copyright (C) 2004 Stephen Strudwick (Pipex Communications, sas@pipex.net)
 *
 * License Information:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *****************************************************************************/

/*
 * Some lines of code are based or taken from the NetSaintService for NT
 * which is Copyright (c) 2000 Jan Christian Kaldestad which as far as I can tell
 * is freeware.
 */

#include "stdafx.h"
#include "stdafx.h"
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "memory.h"

#define _WIN32_WINNT 0x0500

#include <windows.h>

const int OK = 0;
const int Warning = 1;
const int Critical = 2;
const int Unknown = 3;

// Use to change the divisor from Kb to Mb.
#define DIV 1024

char *divisor = "K";

// Handle the width of the field in which to print numbers this way to
// make changes easier. The asterisk in the print format specifier
// "%*ld" takes an int from the argument list, and uses it to pad and
// right-justify the number being formatted.
#define WIDTH 7

int main(int argc, char* argv[])
{
	int freeswap_memory;
	int totalswap_memory;
	int freephys_memory;
	int totalphys_memory;
	int warning_threshold;
	int critical_threshold;	
	MEMORYSTATUS stat;

	/* Get Values from command line */
	if (argc != 3) {
      printf ("NRPE_NT Plugin, MEMORY Load, GX Networks, Stephen Strudwick\n");
      printf ("usage: memload warning threshold(%%) critical threshold(%%)\n");
      printf ("example: 70 90\n");
      return Unknown;
	}

    warning_threshold = atoi(argv[1]);
	critical_threshold = atoi(argv[2]);

    GlobalMemoryStatus (&stat);

	totalphys_memory = stat.dwTotalPhys/1048576;
	freephys_memory = stat.dwAvailPhys/1048576;
	totalswap_memory = stat.dwTotalPageFile/1048576;
	freeswap_memory = stat.dwAvailPageFile/1048576;

	if (!totalswap_memory) {
      return Unknown;
	}
  
	if (!freephys_memory) {
      return Unknown;
	}
  
	if (!totalphys_memory) {
      return Unknown;
	}
  
	if (!freeswap_memory) {
      return Unknown;
	}

	double mean_used = ((double)((totalphys_memory)-(freephys_memory)) / (double)totalphys_memory) * 100;
	double mean_free = ((double)freephys_memory / (double)totalphys_memory) * 100;

	double mean_pused = ((double)((totalswap_memory)-(freeswap_memory)) / (double)totalswap_memory) * 100;
	double mean_pfree = ((double)freeswap_memory / (double)totalswap_memory) * 100;

	printf("Mem: %d MB (%d%%) / %d MB (%d%%) ", ((totalphys_memory)-(freephys_memory)), (int)floor(mean_used), (totalphys_memory), (int)floor(mean_free));

	/* hack, on some systems the paged memory values are reversed !?? So dont print out anything if its fscked */
	if (mean_pused >= 0 && mean_pused <= 100) {
	  printf("Paged Mem: %d MB (%d%%) / %d MB (%d%%)\n", ((totalswap_memory)-(freeswap_memory)), (int)floor(mean_pused), (totalswap_memory), (int)floor(mean_pfree));
    }

	if (mean_used >= critical_threshold)
		return Critical;

	if (mean_used >= warning_threshold)
		return Warning;
	
	return OK;
}