/******************************************************************************
 *
 * Program: Service Check plugin for Nagios (nrpe_nt).
 * License: GPL
 *
 * Designed as a Nagios Plugin (based on plugins by Ethan Galstad).
 * Copyright (C) 2004 Stephen Strudwick (Pipex Communications, sas@pipex.net)
 *
 * License Information:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *****************************************************************************/

#include "stdafx.h"	
#include <stdlib.h>
#include "checkService.h"

DWORD CheckService(const char* servName, DWORD &stat)
{
	SC_HANDLE			schService;
    SC_HANDLE			schSCManager;
	SERVICE_STATUS		ssStatus;
	DWORD				dwRet;
	DWORD				bufSize=0;
	char				*lpServiceName=0;

	stat = -1;

	// Open a handle to the SC Manager database. 
	schSCManager = OpenSCManager( NULL,				// machine (NULL == local)
								  NULL,				// database (NULL == default)
								  GENERIC_READ );	// access required
		
	if ( schSCManager ) 
	{			
		GetServiceKeyName(	schSCManager,	// handle to a service control manager database
							servName,		// the service's display name
							lpServiceName,	// buffer to receive the service name
							&bufSize );		// size of service name buffer and service name

        // OK
		lpServiceName = new char[bufSize+1];
		
		GetServiceKeyName(	schSCManager,	
							servName,		
							lpServiceName,	
							&bufSize );
		
		// Open a handle to the service.
		schService = OpenService( schSCManager,			 // SCM database 
								  lpServiceName,		 // service name
								  SERVICE_INTERROGATE ); // Requests the service to update  
														 // immediately its current status 
														 // information to the service control
														 // manager
		delete [] lpServiceName;
				
		if (schService == NULL)  
		{
			dwRet = GetLastError();
			CloseServiceHandle(schSCManager);			
			return dwRet;						
		}
		
		// Send a interrogate control code to the service.
		if( !ControlService( schService,					// handle to service	 
							 SERVICE_CONTROL_INTERROGATE,   // control code
							 &ssStatus ))					// pointer to service status structure
		{
			dwRet = GetLastError();
			CloseServiceHandle(schSCManager);
			CloseServiceHandle(schService);			
			return dwRet;			
		}
	}
	else
		return GetLastError();
	
	stat = ssStatus.dwCurrentState;

	CloseServiceHandle(schSCManager);
	CloseServiceHandle(schService);
	
	return ERROR_SUCCESS;
}

int CheckServices(char *services) {	
	DWORD	stat;
	DWORD	err;
	int rc = OK;

	/* split and loop over services */
	int n = 0;
	int ptr = 0;
	int len = strlen(services);
	char buf[257];

	/* Loop until NULL */
	for(n=0;n<len+1;n++) {
		/* Go till we hit a comma */
		if (services[n] != ',' && services[n] != '\0') {
			buf[ptr] = services[n];
			ptr++;
		} else if (ptr) {
			/* Got null terminated string */
			buf[ptr] = '\0';
			/* Reset pointer */
			ptr = 0;
			
			err = CheckService(buf, stat);
			if(err != 0) {
				rc = Critical;
				switch(err)
					  {
					case ERROR_INVALID_NAME:
						printf("%s CRITICAL(INVALID_NAME) ", buf);
						break;
					case ERROR_SERVICE_DOES_NOT_EXIST:
						printf("%s CRITICAL(DOES_NOT_EXIST) ", buf);
						break;
					case ERROR_SERVICE_NOT_ACTIVE:
						printf("%s CRITICAL(NOT_ACTIVE) ", buf);
						break;
					case ERROR_SERVICE_REQUEST_TIMEOUT:
						printf("%s CRITICAL(REQUEST_TIMEOUT) ", buf);
						break;
					  default:
						  break;
					  }
			} else {
				switch(stat)
					  {
					  case SERVICE_RUNNING:
						  printf("%s OK ", buf);
						  break;
					  case SERVICE_START_PENDING:
						  if (rc != Critical)
							rc = Warning;
						  printf("%s WARNING(START_PENDING) ", buf);
						  break;
					  case SERVICE_PAUSED:
						  if (rc != Critical)
							rc = Warning;
						  printf("%s WARNING(PAUSED) ", buf);
						  break;
					  case SERVICE_PAUSE_PENDING:
						  if (rc != Critical)
							rc = Warning;
						  printf("%s WARNING(PAUSE_PENDING) ", buf);
						  break;
					  default:
						  break;
					}
			}
		}
	}

	return rc;
} 

