/******************************************************************************
 *
 * Program: Service Check plugin for Nagios (nrpe_nt).
 * License: GPL
 *
 * Designed as a Nagios Plugin (based on plugins by Ethan Galstad).
 * Copyright (C) 2004 Stephen Strudwick (Pipex Communications, sas@pipex.net)
 *
 * License Information:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *****************************************************************************/

#ifndef _checkServiceH
#define _checkServiceH

#include "windows.h"
#include <stdio.h>
#include "common.h"

// Valid status codes for a service, returned to the client
#define	SERV_SERVICE_DOES_NOT_EXIST		1
#define	SERV_SERVICE_NOT_ACTIVE			2
#define	SERV_SERVICE_REQUEST_TIMEOUT	3
#define	SERV_CHECK_ERROR				4
#define	SERV_PAUSED						5
#define SERV_RUNNING					6

//
//  FUNCTION: DWORD CheckService(char *servName, DWORD &status)
//
//  PURPOSE: Check the current status of the specified service 
//
//  PARAMETERS:
//    servName - the display name of the service (case sensitive!)
//    stat - the status of the service. 
//			Following states are possible:
//				SERVICE_STOPPED				The service is not running. 
//				SERVICE_START_PENDING		The service is starting. 
//				SERVICE_STOP_PENDING		The service is stopping. 
//				SERVICE_RUNNING				The service is running. 
//				SERVICE_CONTINUE_PENDING	The service continue is pending. 
//				SERVICE_PAUSE_PENDING		The service pause is pending. 
//				SERVICE_PAUSED				The service is paused.    
//
//  RETURN VALUES:
//		ERROR_SUCCESS						The operation completed successfully.
//		ERROR_ACCESS_DENIED 				The specified service control manager 
//											database handle does not have access 
//											to the service.
//		ERROR_INVALID_HANDLE 				The specified handle is invalid.
//		ERROR_INVALID_NAME					The specified service name is invalid. 
//		ERROR_SERVICE_DOES_NOT_EXIST		The specified service does not exist.
//		ERROR_DEPENDENT_SERVICES_RUNNING	The service cannot be stopped 
//											because other running services are 
//											dependent on it. 
//		ERROR_INVALID_SERVICE_CONTROL		The requested control code is not valid, 
//											or it is unacceptable to the service. 
//		ERROR_SERVICE_CANNOT_ACCEPT_CTRL	The requested control code cannot be
//											sent to the service because the 
//											state of the service is SERVICE_STOPPED,
//											SERVICE_START_PENDING, or SERVICE_STOP_PENDING.
//		ERROR_SERVICE_NOT_ACTIVE			The service has not been started.
//		ERROR_SERVICE_REQUEST_TIMEOUT		The service did not respond to the start 
//											request in a timely fashion.
//		ERROR_DATABASE_DOES_NOT_EXIST		The specified database does not exist. 
//		ERROR_INVALID_PARAMETER				A specified parameter is invalid.
//
DWORD CheckService(const char *servName, DWORD &status);
int CheckServices(char *services);

#endif